# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from django import template
from django.template.defaultfilters import stringfilter


register = template.Library()


@register.filter(name='iconset_from_class')
@stringfilter
def iconset_from_class(value):
    """
    extracts the iconset from a class definition
    "fa-flask" -> "fa"
    :param value:
    :return:
    """
    if '-' in value:
        return value.split('-')[0]
    return ''



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from django.utils.translation import ugettext_lazy as _
import django.forms.widgets

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from . import models, forms, constants, widgets
from cms.models import CMSPlugin


link_fieldset = (
    ('Link', {
        'fields': (
            'link_page', 'link_file', 'link_url', 'link_mailto', 'link_phone',
        ),
        'description': 'Choose one of the link types below.',
    }),
    ('Link options', {
        'fields': (
            ('link_target', 'link_anchor',),
        ),
    }),
)


#################
# Basic Plugins #
#################


class Bootstrap3BlockquoteCMSPlugin(CMSPluginBase):
    model = models.Boostrap3BlockquotePlugin
    name = _("Blockquote")
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/blockquote/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/blockquote.html'
    allow_children = True

    def render(self, context, instance, placeholder):
        context.update({'instance': instance})
        return context


plugin_pool.register_plugin(Bootstrap3BlockquoteCMSPlugin)


class Bootstrap3IconCMSPlugin(CMSPluginBase):
    model = models.Boostrap3IconPlugin
    name = _("Icon")
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/icon/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/icon.html'
    text_enabled = True
    allow_children = True

    fieldsets = (
        (None, {'fields': (
            'icon',
        )}),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context.update({'instance': instance})
        return context


plugin_pool.register_plugin(Bootstrap3IconCMSPlugin)


class Bootstrap3LabelCMSPlugin(CMSPluginBase):
    model = models.Boostrap3LabelPlugin
    name = _("Label")
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/label/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/label.html'
    text_enabled = True
    allow_children = True

    fieldsets = (
        (None, {'fields': (
            'label',
            'context',
        )}),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context.update({'instance': instance})
        return context


plugin_pool.register_plugin(Bootstrap3LabelCMSPlugin)


class Bootstrap3WellCMSPlugin(CMSPluginBase):
    model = models.Boostrap3WellPlugin
    name = _("Well")
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/well/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/well.html'
    allow_children = True

    fieldsets = (
        (None, {'fields': (
            'size',
        )}),

        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context.update({'instance': instance})
        return context


plugin_pool.register_plugin(Bootstrap3WellCMSPlugin)


class Bootstrap3AlertCMSPlugin(CMSPluginBase):
    model = models.Boostrap3AlertPlugin
    name = _("Alert")
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/alert/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/alert.html'
    allow_children = True

    fieldsets = (
        (None, {'fields': (
            'context',
            'icon',
        )}),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context.update({'instance': instance})
        return context


plugin_pool.register_plugin(Bootstrap3AlertCMSPlugin)


class Bootstrap3ButtonCMSPlugin(CMSPluginBase):
    model = models.Boostrap3ButtonPlugin
    name = _("Link/Button")
    module = _('Bootstrap3')
    form = forms.LinkForm
    change_form_template = 'admin/aldryn_bootstrap3/plugins/button/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/button.html'
    text_enabled = True
    allow_children = True

    fieldsets = (
        (None, {
            'fields': (
                'label',
                'type',
                'btn_context',
                'btn_size',
                'btn_block',
                'txt_context',
                'icon_left',
                'icon_right',
            ),
        }),
    ) + link_fieldset + (
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
                # 'responsive',
                # 'responsive_print',
            )
        }),
    )

    def render(self, context, instance, placeholder):
        context.update({'instance': instance})
        return context

    def icon_src(self, instance):
        from django.contrib.staticfiles.templatetags.staticfiles import static

        return static("cms/img/icons/plugins/link.png")


plugin_pool.register_plugin(Bootstrap3ButtonCMSPlugin)


class Bootstrap3ImageCMSPlugin(widgets.BootstrapMediaMixin, CMSPluginBase):
    model = models.Boostrap3ImagePlugin
    name = _("Image")
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/image/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/image.html'
    allow_children = True
    cache = False

    fieldsets = (
        (None, {'fields': (
                'file',
                'aspect_ratio',
                'shape',
                'thumbnail',
                'alt',
        )}),

        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'title',
                'classes',
                'img_responsive',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context.update({'instance': instance})
        return context


plugin_pool.register_plugin(Bootstrap3ImageCMSPlugin)


class Bootstrap3SpacerCMSPlugin(CMSPluginBase):
    model = models.Boostrap3SpacerPlugin
    name = _("Spacer")
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/spacer/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/spacer.html'

    fieldsets = (
        (None, {'fields': (
            'size',
        )}),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context.update({'instance': instance})
        return context


plugin_pool.register_plugin(Bootstrap3SpacerCMSPlugin)


class Bootstrap3FileCMSPlugin(CMSPluginBase):
    model = models.Bootstrap3FilePlugin
    name = _("File")
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/file/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/file.html'

    fieldsets = (
        (None, {'fields': (
            'file',
            'name',
            'icon_left',
            'icon_right',
        )}),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'open_new_window',
                'show_file_size',
                'classes',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context.update({'instance': instance})
        return context

plugin_pool.register_plugin(Bootstrap3FileCMSPlugin)


#########
# Panel #
#########


class Bootstrap3PanelCMSPlugin(CMSPluginBase):
    model = models.Boostrap3PanelPlugin
    name = _("Panel")
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/panel/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/panel.html'
    form = forms.PanelPluginBaseForm
    allow_children = True

    fieldsets = (
        ('Create', {
            'fields': (
                ('create_heading', 'create_body', 'create_footer'),
            )
        }),
        (None, {'fields': (
            'context',
        )}),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context.update({'instance': instance})
        return context

    def save_model(self, request, obj, form, change):
        response = super(Bootstrap3PanelCMSPlugin, self).save_model(request, obj, form, change)
        data = form.cleaned_data
        extra = {}
        subplugins = (
            ('create_heading', models.Boostrap3PanelHeadingPlugin, Bootstrap3PanelHeadingCMSPlugin),
            ('create_body', models.Boostrap3PanelBodyPlugin, Bootstrap3PanelBodyCMSPlugin),
            ('create_footer', models.Boostrap3PanelFooterPlugin, Bootstrap3PanelFooterCMSPlugin),
        )
        existing_plugins = [p.plugin_type for p in obj.get_children()]
        for field, model_class, plugin_class in subplugins:
            if not data.get(field):
                # TODO: delete?
                continue
            if plugin_class.__name__ in existing_plugins:
                continue
            plugin = model_class(
                parent=obj,
                placeholder=obj.placeholder,
                language=obj.language,
                position=CMSPlugin.objects.filter(parent=obj).count(),
                plugin_type=plugin_class.__name__,
                **extra
            )
            plugin.save()
        return response


plugin_pool.register_plugin(Bootstrap3PanelCMSPlugin)


class Bootstrap3PanelHeadingCMSPlugin(CMSPluginBase):
    model = models.Boostrap3PanelHeadingPlugin
    name = _("Panel Heading")
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/panel_heading/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/panel_heading.html'
    allow_children = True
    parent_classes = ['Bootstrap3PanelCMSPlugin']

    fieldsets = (
        (None, {'fields': (
            'title',
        )}),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context.update({'instance': instance})
        return context


plugin_pool.register_plugin(Bootstrap3PanelHeadingCMSPlugin)


class Bootstrap3PanelBodyCMSPlugin(CMSPluginBase):
    model = models.Boostrap3PanelBodyPlugin
    name = _("Panel Body")
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/panel_body/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/panel_body.html'
    allow_children = True
    parent_classes = ['Bootstrap3PanelCMSPlugin']

    fieldsets = (
        # (None, {'fields': (
        # 'title',
        # )}),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context.update({'instance': instance})
        return context


plugin_pool.register_plugin(Bootstrap3PanelBodyCMSPlugin)


class Bootstrap3PanelFooterCMSPlugin(CMSPluginBase):
    model = models.Boostrap3PanelFooterPlugin
    name = _("Panel Footer")
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/panel_footer/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/panel_footer.html'
    allow_children = True
    parent_classes = ['Bootstrap3PanelCMSPlugin']

    fieldsets = (
        # (None, {'fields': (
        # 'title',
        # )}),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context.update({'instance': instance})
        return context


plugin_pool.register_plugin(Bootstrap3PanelFooterCMSPlugin)


########
# Grid #
########


class Bootstrap3RowCMSPlugin(widgets.BootstrapMediaMixin, CMSPluginBase):
    model = models.Bootstrap3RowPlugin
    name = _('Row')
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/row/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/row.html'
    allow_children = True
    child_classes = ['Bootstrap3ColumnCMSPlugin']
    form = forms.RowPluginForm
    fieldsets = [
        ("Create Columns", {
            # 'classes': ('collapse',),
            'fields': (
                          'create',
                      ) + tuple([
                (
                    'create_{}_col'.format(size),
                    'create_{}_offset'.format(size),
                    'create_{}_push'.format(size),
                    'create_{}_pull'.format(size),
                ) for size in constants.DEVICE_SIZES
            ])
        }),
        ("Advanced", {
            'fields': (
                'classes',
            )
        }),
    ]

    def render(self, context, instance, placeholder):
        context = super(Bootstrap3RowCMSPlugin, self).render(context, instance, placeholder)
        return context

    def save_model(self, request, obj, form, change):
        response = super(Bootstrap3RowCMSPlugin, self).save_model(request, obj, form, change)
        data = form.cleaned_data
        for x in xrange(int(data['create']) if data['create'] is not None else 0):
            extra = {}
            for size in constants.DEVICE_SIZES:
                for element in ['col', 'offset', 'push', 'pull']:
                    extra['{}_{}'.format(size, element)] = data.get(
                        'create_{}_{}'.format(size, element)) or None
            col = models.Bootstrap3ColumnPlugin(
                parent=obj,
                placeholder=obj.placeholder,
                language=obj.language,
                position=CMSPlugin.objects.filter(parent=obj).count(),
                plugin_type=Bootstrap3ColumnCMSPlugin.__name__,
                **extra
            )
            col.save()
        return response


class Bootstrap3ColumnCMSPlugin(CMSPluginBase, widgets.BootstrapMediaMixin):
    model = models.Bootstrap3ColumnPlugin
    name = _('Column')
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/column/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/column.html'
    allow_children = True
    parent_classes = ['Bootstrap3RowCMSPlugin']

    fieldsets = [
        (None, {
            'fields': tuple([
                (
                    '{}_col'.format(size),
                    '{}_offset'.format(size),
                    '{}_push'.format(size),
                    '{}_pull'.format(size),
                ) for size in constants.DEVICE_SIZES
            ])
        }),
        ("Advanced", {
            'classes': ('collapse',),
            'fields': (
                'classes',
                'tag',
            )
        }),
    ]


plugin_pool.register_plugin(Bootstrap3RowCMSPlugin)
plugin_pool.register_plugin(Bootstrap3ColumnCMSPlugin)


#############
# Accordion #
#############


class Bootstrap3AccordionCMSPlugin(CMSPluginBase, widgets.BootstrapMediaMixin):
    model = models.Bootstrap3AccordionPlugin
    name = _('Accordion')
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/accordion/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/accordion.html'
    allow_children = True
    child_classes = ['Bootstrap3AccordionItemCMSPlugin']

    fieldsets = (
        (None, {
            'fields': (
                'index',
            )
        }),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context = super(Bootstrap3AccordionCMSPlugin, self).render(context, instance, placeholder)
        context['instance'] = instance
        context['accordion'] = instance
        context['accordion_id'] = "plugin-bootstrap3-accordion-%s" % instance.pk
        return context


class Bootstrap3AccordionItemCMSPlugin(CMSPluginBase, widgets.BootstrapMediaMixin):
    model = models.Bootstrap3AccordionItemPlugin
    name = _('Accordion item')
    module = _('Accordion')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/accordion_item/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/accordion_item.html'
    allow_children = True
    parent_classes = ['Bootstrap3AccordionCMSPlugin']
    cache = False

    fieldsets = (
        (None, {
            'fields': (
                'title',
                'context',
            )
        }),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context = super(Bootstrap3AccordionItemCMSPlugin, self).render(context, instance, placeholder)
        context['instance'] = instance
        context['item'] = instance
        return context


plugin_pool.register_plugin(Bootstrap3AccordionCMSPlugin)
plugin_pool.register_plugin(Bootstrap3AccordionItemCMSPlugin)


#############
# ListGroup #
#############


class Bootstrap3ListGroupCMSPlugin(CMSPluginBase, widgets.BootstrapMediaMixin):
    model = models.Bootstrap3ListGroupPlugin
    name = _('List Group')
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/list_group/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/list_group.html'
    allow_children = True
    child_classes = ['Bootstrap3ListGroupItemCMSPlugin']

    fieldsets = (
        # (None, {
        #     'fields': (
        #         'index',
        #     )
        # }),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
                'add_list_group_class',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context = super(Bootstrap3ListGroupCMSPlugin, self).render(context, instance, placeholder)
        context['instance'] = instance
        return context


class Bootstrap3ListGroupItemCMSPlugin(CMSPluginBase, widgets.BootstrapMediaMixin):
    model = models.Bootstrap3ListGroupItemPlugin
    name = _('List Group Item')
    module = _('Bootstrap3')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/list_group_item/change_form.html'
    render_template = 'aldryn_bootstrap3/plugins/list_group_item.html'
    allow_children = True
    parent_classes = [
        'Bootstrap3ListGroupCMSPlugin',
        'Bootstrap3PanelCMSPlugin',
    ]
    cache = False

    fieldsets = (
        (None, {
            'fields': (
                'title',
                'context',
                'state',
            )
        }),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context = super(Bootstrap3ListGroupItemCMSPlugin, self).render(context, instance, placeholder)
        context['instance'] = instance
        context['item'] = instance
        return context


plugin_pool.register_plugin(Bootstrap3ListGroupCMSPlugin)
plugin_pool.register_plugin(Bootstrap3ListGroupItemCMSPlugin)


############
# Carousel #
############


# Base Classes
class CarouselBase(CMSPluginBase, widgets.BootstrapMediaMixin):
    module = _('Bootstrap3')


class CarouselSlideBase(CarouselBase, widgets.BootstrapMediaMixin):
    require_parent = True
    parent_classes = ['Bootstrap3CarouselCMSPlugin']

    def render(self, context, instance, placeholder):
        # get style from parent plugin, render chosen template
        context['instance'] = instance
        context['image'] = instance.image
        return context

    def get_slide_template(self, instance, name='slide'):
        if instance.parent is None:
            style = models.Bootstrap3CarouselPlugin.STYLE_DEFAULT
        else:
            style = getattr(
                instance.parent.get_plugin_instance()[0],
                'style',
                models.Bootstrap3CarouselPlugin.STYLE_DEFAULT,
            )
        return 'aldryn_bootstrap3/plugins/carousel/{}/{}.html'.format(
            style, name)

    def get_render_template(self, context, instance, placeholder):
        return self.get_slide_template(instance=instance)


# Plugins
class Bootstrap3CarouselCMSPlugin(CarouselBase):
    name = _('Carousel')
    model = models.Bootstrap3CarouselPlugin
    change_form_template = 'admin/aldryn_bootstrap3/plugins/carousel/change_form.html'
    render_template = False
    form = forms.CarouselPluginForm
    allow_children = True
    cache = False
    child_classes = [
        'Bootstrap3CarouselSlideCMSPlugin',
        # 'Bootstrap3CarouselSlideFolderCMSPlugin',
    ]
    fieldsets = (
        (None, {
            'fields': (
                'style',
                'transition_effect',
                ('ride', 'interval'),
                'aspect_ratio',

            )
        }),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
                'pause',
                'wrap',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        if instance.child_plugin_instances:
            number_of_slides = sum([
                plugin.folder.file_count
                if isinstance(plugin, Bootstrap3CarouselCMSPlugin) else 1
                for plugin in instance.child_plugin_instances
            ])
        else:
            number_of_slides = 0
        context['slides'] = range(number_of_slides)
        return context

    def get_render_template(self, context, instance, placeholder):
        return 'aldryn_bootstrap3/plugins/carousel/{}/carousel.html'.format(
            instance.style)


carousel_link_fieldset = link_fieldset


class Bootstrap3CarouselSlideCMSPlugin(CarouselSlideBase):
    model = models.Bootstrap3CarouselSlidePlugin
    name = _('Carousel Slide')
    change_form_template = 'admin/aldryn_bootstrap3/plugins/carousel_slide/change_form.html'
    allow_children = True
    cache = False
    fieldsets = (
        (None, {
            'fields': (
                'image',
                'content',
            )
        }),
    ) + link_fieldset + (
        (_('Link Text'), {
            'fields': (
                'link_text',
            )
        }),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
            ),
        }),
    )


class Bootstrap3CarouselSlideFolderCMSPlugin(CarouselSlideBase):
    """
    Slide Plugin that renders a slide for each image in the linked folder.
    """
    name = _('Carousel Slides Folder')
    model = models.Bootstrap3CarouselSlideFolderPlugin
    cache = False

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        context['slide_template'] = self.get_slide_template(
            instance=instance,
            name='image_slide',
        )
        return context

    def get_render_template(self, context, instance, placeholder):
        return self.get_slide_template(instance=instance, name='slide_folder')


plugin_pool.register_plugin(Bootstrap3CarouselCMSPlugin)
plugin_pool.register_plugin(Bootstrap3CarouselSlideCMSPlugin)
# plugin_pool.register_plugin(Bootstrap3CarouselSlideFolderCMSPlugin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import django.forms
import django.forms.models
import django.core.exceptions
import django.template
import django.template.loader
from django.utils.translation import ugettext_lazy as _
import cms.forms.fields
import cms.models
from django.forms.widgets import Media

from . import models, constants


class RowPluginBaseForm(django.forms.models.ModelForm):
    create = django.forms.IntegerField(
        label=_('Create Columns'),
        help_text=_('Create this number of columns inside'),
        required=False,
        min_value=0,
    )

    class Meta:
        model = models.Bootstrap3RowPlugin
        # fields = ('classes',)
        exclude = ('page', 'position', 'placeholder', 'language', 'plugin_type')


extra_fields = {}
for size, name in constants.DEVICE_CHOICES:
    extra_fields["create_{}_col".format(size)] = django.forms.IntegerField(
        label=_('col-{}-'.format(size)),
        help_text=('Width of created columns. You can still change the width of the column afterwards.'),
        required=False,
        min_value=1,
        max_value=constants.GRID_SIZE,
    )
    extra_fields["create_{}_offset".format(size)] = django.forms.IntegerField(
        label=_('offset-'.format(size)),
        help_text=('Offset of created columns. You can still change the width of the column afterwards.'),
        required=False,
        min_value=1,
        max_value=constants.GRID_SIZE,
    )
    extra_fields["create_{}_push".format(size)] = django.forms.IntegerField(
        label=_('push-'.format(size)),
        help_text=('Push of created columns. You can still change the width of the column afterwards.'),
        required=False,
        min_value=1,
        max_value=constants.GRID_SIZE,
    )
    extra_fields["create_{}_pull".format(size)] = django.forms.IntegerField(
        label=_('pull-'.format(size)),
        help_text=('Pull of created columns. You can still change the width of the column afterwards.'),
        required=False,
        min_value=1,
        max_value=constants.GRID_SIZE,
    )


RowPluginForm = type(str('RowPluginBaseForm'), (RowPluginBaseForm,), extra_fields)


class LinkForm(django.forms.models.ModelForm):
    link_page = cms.forms.fields.PageSelectFormField(
        queryset=cms.models.Page.objects.drafts(),
        label=_("Page"),
        required=False,
    )

    def for_site(self, site):
        # override the page_link fields queryset to containt just pages for
        # current site
        self.fields['link_page'].queryset = cms.models.Page.objects.drafts().on_site(site)

    class Meta:
        model = models.Boostrap3ButtonPlugin
        exclude = (
            'page', 'position', 'placeholder', 'language', 'plugin_type',
        )

    def _get_media(self):
        """
        Provide a description of all media required to render the widgets on this form
        """
        media = Media()
        for field in self.fields.values():
            media = media + field.widget.media
        media._js = ['cms/js/libs/jquery.min.js'] + media._js
        return media
    media = property(_get_media)

    def clean(self):
        cleaned_data = super(LinkForm, self).clean()
        link_fields = {
            'link_url': cleaned_data.get("link_url"),
            'link_page': cleaned_data.get("link_page"),
            'link_file': cleaned_data.get("link_file"),
            'link_mailto': cleaned_data.get("link_mailto"),
            'link_phone': cleaned_data.get("link_phone"),
        }
        error_msg = _("Only one of Page, File, Link, Email address or Phone is allowed.")
        if len([i for i in link_fields.values() if i]) > 1:
            for field, value in link_fields.items():
                if value:
                    self._errors[field] = self.error_class([error_msg])
            raise django.core.exceptions.ValidationError(error_msg)
        return cleaned_data


class PanelPluginBaseForm(django.forms.models.ModelForm):
    create_heading = django.forms.BooleanField(required=False, initial=False)
    create_body = django.forms.BooleanField(required=False, initial=False)
    create_footer = django.forms.BooleanField(required=False, initial=False)

    class Meta:
        model = models.Boostrap3PanelPlugin
        # fields = ('classes',)
        exclude = ('page', 'position', 'placeholder', 'language', 'plugin_type')


class CarouselPluginForm(django.forms.ModelForm):

    class Meta:
        fields = [
            'style',
            'transition_effect',
            'ride',
            'interval',
        ]
        model = models.Bootstrap3CarouselPlugin

    def clean_style(self):
        style = self.cleaned_data.get('style')
        template = 'aldryn_bootstrap3/plugins/carousel/{}/carousel.html'.format(
            style)
        print style
        # Check if template for style exists:
        try:
            django.template.loader.select_template([template])
        except django.template.TemplateDoesNotExist:
            raise django.forms.ValidationError(
                _("Not a valid style (Template %s does not exist)") % template
            )
        return style



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from django.utils.translation import ugettext_lazy as _
from .conf import settings

CONTEXT_CHOICES = (
    ('primary', 'Primary',),
    ('success', 'Success',),
    ('info', 'Info',),
    ('warning', 'Warning',),
    ('danger', 'Danger',),
)
CONTEXT_DEFAULT = 'default'

BUTTON_CONTEXT_CHOICES = (
    ('default', 'Default',),
) + CONTEXT_CHOICES + (
    ('link', 'Link Button',),
    ('', 'Custom',),
)
BUTTON_CONTEXT_DEFAULT = 'default'

TXT_LINK_CONTEXT_CHOICES = (
    ('', 'None',),
) + CONTEXT_CHOICES + (
    # ('alert-link', 'Alert Link',),
    ('muted', 'Muted',),
)
TXT_LINK_CONTEXT_DEFAULT = ''

LABEL_CONTEXT_CHOICES = (
    ('default', 'Default',),
) + CONTEXT_CHOICES + (
    # ('', 'Custom',),
)
LABEL_CONTEXT_DEFAULT = 'default'

PANEL_CONTEXT_CHOICES = (
    ('default', 'Default',),
) + CONTEXT_CHOICES + (
    # ('', 'Custom',),
)
PANEL_CONTEXT_DEFAULT = 'default'

ACCORDION_ITEM_CONTEXT_CHOICES = PANEL_CONTEXT_CHOICES
ACCORDION_ITEM_CONTEXT_DEFAULT = PANEL_CONTEXT_DEFAULT

LIST_GROUP_ITEM_CONTEXT_CHOICES = (
    ('', 'None',),
) + CONTEXT_CHOICES
LIST_GROUP_ITEM_CONTEXT_DEFAULT = ''

SIZE_CHOICES = (
    ('lg', 'Large',),
    ('md', 'Medium',),
    ('sm', 'Small',),
    ('xs', 'Extra Small',),
)

SIZE_WIDGET_CHOICES = (
    # ('', 'Default'),
) + SIZE_CHOICES
SIZE_WIDGET_DEFAULT = 'md'

SIZES = tuple([size for size, name in SIZE_CHOICES])

SIZE_DEFAULT = 'md'


# WARNING: changing DEVICE_CHOICES identifier will cause model creation to change and
#          requires database migrations!
DEVICES = (
    {
        'identifier': 'xs',
        'name': _("mobile phones"),
        'width': 768,
        'width_gutter': 750,
        'icon': 'mobile-phone',
    },
    {
        'identifier': 'sm',
        'name': _("tablets"),
        'width': 768,
        'width_gutter': 750,
        'icon': 'tablet',
    },
    {
        'identifier': 'md',
        'name': _("laptops"),
        'width': 992,
        'width_gutter': 970,
        'icon': 'laptop',
    },
    {
        'identifier': 'lg',
        'name': _("large desktops"),
        'width': 1200,
        'width_gutter': 1170,
        'icon': 'desktop',
    },
)
for device in DEVICES:
    identifier = device['identifier']
    device['long_description'] = "{name} (<{width}px)".format(**device)
    device['size_name'] = dict(SIZE_CHOICES).get(identifier)

DEVICE_DICT = {device['identifier']: device for device in DEVICES}

DEVICE_CHOICES = (
    ('xs', _("Tiny")),
    ('sm', _("Small")),
    ('md', _("Medium")),
    ('lg', _("Large")),
)
DEVICE_SIZES = tuple([size for size, name in DEVICE_CHOICES])

GRID_SIZE = settings.ALDRYN_BOOTSTRAP3_GRID_SIZE


ASPECT_RATIOS = (
    (1, 1),
    (4, 3),
    (16, 9),
    (16, 10),
    (21, 9),
)
ASPECT_RATIOS_REVERSED = tuple([(y, x) for x, y in ASPECT_RATIOS])

ASPECT_RATIO_CHOICES = \
    tuple([
        ('{}x{}'.format(x, y), '{}x{}'.format(x, y))
        for x, y in ASPECT_RATIOS
    ]) + tuple([
        ('{}x{}'.format(x, y), '{}x{}'.format(x, y))
        for x, y in ASPECT_RATIOS_REVERSED
    ])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from django.utils.translation import ugettext_lazy as _
import django.forms.widgets

from .conf import settings
from . import constants


class BootstrapMediaMixin(object):
    class Media:
        css = {
            'all': (
                '//maxcdn.bootstrapcdn.com/bootstrap/3.3.2/css/bootstrap.min.css',
                'https://static.aldryn.com/cdn/bootstrap-iconpicker.min.css',
                '//maxcdn.bootstrapcdn.com/font-awesome/4.2.0/css/font-awesome.min.css',
            )
        }
        js = (
                'https://code.jquery.com/jquery-1.10.2.min.js',
                '//maxcdn.bootstrapcdn.com/bootstrap/3.3.2/js/bootstrap.min.js',
                'https://static.aldryn.com/cdn/iconset/iconset-glyphicon.min.js',
                'https://static.aldryn.com/cdn/iconset/iconset-fontawesome-4.2.0.min.js',
                'https://static.aldryn.com/cdn/bootstrap-iconpicker.min.js',
        )


class ContextRenderer(django.forms.widgets.RadioFieldRenderer):
    def render(self):
        from django.template.loader import render_to_string
        rendered = render_to_string(
            'admin/aldryn_bootstrap3/widgets/context.html',
            {'selects': self},
        )
        return rendered


class Context(BootstrapMediaMixin, django.forms.widgets.RadioSelect):
    renderer = ContextRenderer


class SizeRenderer(BootstrapMediaMixin, django.forms.widgets.RadioFieldRenderer):
    def render(self):
        from django.template.loader import render_to_string
        rendered = render_to_string(
            'admin/aldryn_bootstrap3/widgets/size.html',
            {'selects': self},
        )
        return rendered


class Size(BootstrapMediaMixin, django.forms.widgets.RadioSelect):
    renderer = SizeRenderer


class Icon(BootstrapMediaMixin, django.forms.widgets.TextInput):
    def render(self, name, value, attrs=None, **kwargs):
        input_html = super(Icon, self).render(name, value, attrs=attrs, **kwargs)
        if value is None:
            value = ''
        iconset = value.split('-')[0] if value and '-' in value else ''
        iconset_prefexes = [s[1] for s in settings.ALDRYN_BOOTSTRAP3_ICONSETS]
        if len(settings.ALDRYN_BOOTSTRAP3_ICONSETS) and iconset not in iconset_prefexes:
            # invalid iconset! maybe because the iconset was removed from
            # the project. set it to the first in the list.
            iconset = settings.ALDRYN_BOOTSTRAP3_ICONSETS[0][1]
        from django.template.loader import render_to_string
        rendered = render_to_string(
            'admin/aldryn_bootstrap3/widgets/icon.html',
            {
                'input_html': input_html,
                'value': value,
                'name': name,
                'iconset': iconset,
                'is_required': self.is_required,
                'iconsets': settings.ALDRYN_BOOTSTRAP3_ICONSETS,
            },
        )
        return rendered


class MiniTextarea(django.forms.widgets.Textarea):
    def __init__(self, attrs=None):
        if attrs is None:
            attrs = {}
        attrs['cols'] = '120'
        attrs['rows'] = '1'
        super(MiniTextarea, self).__init__(attrs)


class LinkOrButtonRenderer(django.forms.widgets.RadioFieldRenderer):
    def render(self):
        from django.template.loader import render_to_string
        rendered = render_to_string(
            'admin/aldryn_bootstrap3/widgets/link_or_button.html',
            {'selects': self},
        )
        return rendered


class LinkOrButton(BootstrapMediaMixin, django.forms.widgets.RadioSelect):
    renderer = LinkOrButtonRenderer


class Responsive(BootstrapMediaMixin, django.forms.widgets.Textarea):
    def render(self, name, value, attrs=None):
        from django.template.loader import render_to_string
        widget_html = super(Responsive, self).render(name=name, value=value, attrs=attrs)

        choices = []
        options = (
            ('', 'Pretty None'),
            ('visible-{device}-block', 'Pretty visible-{device}-block'),
            ('visible-{device}-inline', 'Pretty visible-{device}-inline'),
            ('visible-{device}-inline-block', 'Pretty visible-{device}-inline-block'),
            ('hidden-{device}', 'Pretty hidden-{device}'),
        )
        for device in constants.DEVICE_SIZES:
            choices.append((
                device,
                tuple([
                    (option.format(device=device), option_verbose.format(device=device))
                    for option, option_verbose in options
                ])
            ))
        choices = tuple(choices)


        rendered = render_to_string(
            'admin/aldryn_bootstrap3/widgets/responsive.html',
            {
                'widget_html': widget_html,
                'widget': self,
                'value': value,
                'name': name,
                'id': attrs.get('id', None),
                'attrs': attrs,
                'choices': choices,
            },
        )
        return rendered


class ResponsivePrint(BootstrapMediaMixin, django.forms.widgets.Textarea):
    def render(self, name, value, attrs=None):
        from django.template.loader import render_to_string
        widget_html = super(ResponsivePrint, self).render(
            name=name, value=value, attrs=attrs)

        choices = [
            ('visible-print-block', 'Pretty visible-print-block'),
            ('visible-print-inline', 'Pretty visible-print-inline'),
            ('visible-print-inline-block', 'Pretty visible-print-inline-block'),
            ('hidden-print', 'Pretty visible-print-inline-block'),
        ]
        rendered = render_to_string(
            'admin/aldryn_bootstrap3/widgets/responsive_print.html',
            {
                'widget_html': widget_html,
                'widget': self,
                'value': value,
                'name': name,
                'id': attrs.get('id', None),
                'attrs': attrs,
                'choices': choices,
            },
        )
        return rendered



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import filer.fields.file
import aldryn_bootstrap3.model_fields
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0002_auto_20150606_2003'),
        ('cms', '0011_auto_20150419_1006'),
        ('aldryn_bootstrap3', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Bootstrap3FilePlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, related_name='+', primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('name', aldryn_bootstrap3.model_fields.MiniText(default='', verbose_name='name', blank=True)),
                ('open_new_window', models.BooleanField(default=False)),
                ('show_file_size', models.BooleanField(default=False)),
                ('icon_left', aldryn_bootstrap3.model_fields.Icon(default='', max_length=255, blank=True)),
                ('icon_right', aldryn_bootstrap3.model_fields.Icon(default='', max_length=255, blank=True)),
                ('classes', aldryn_bootstrap3.model_fields.Classes(default='', help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>', blank=True)),
                ('file', filer.fields.file.FilerFileField(related_name='+', on_delete=django.db.models.deletion.SET_NULL, verbose_name='file', to='filer.File', null=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]
